
/* Chrysalide - Outil d'analyse de fichiers binaires
 * register-int.h - définitions internes pour la représentation générique d'un registre
 *
 * Copyright (C) 2012-2013 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _ARCH_REGISTER_INT_H
#define _ARCH_REGISTER_INT_H


#include "register.h"


#include "operand-int.h"
#include "sharing/instance-int.h"



/* ---------------------------- PUR REGISTRE DU MATERIEL ---------------------------- */


/* Produit une empreinte à partir d'un registre. */
typedef guint (* reg_hash_fc) (const GArchRegister *);

/* Compare un registre avec un autre. */
typedef int (* reg_compare_fc) (const GArchRegister *, const GArchRegister *);

/* Traduit un registre en version humainement lisible. */
typedef void (* reg_print_fc) (const GArchRegister *, GBufferLine *, AsmSyntax);

/* Indique si le registre correspond à ebp ou similaire. */
typedef bool (* reg_is_base_pointer_fc) (const GArchRegister *);

/* Indique si le registre correspond à esp ou similaire. */
typedef bool (* reg_is_stack_pointer_fc) (const GArchRegister *);



/* Représentation d'un registre (instance) */
struct _GArchRegister
{
    GObject parent;                         /* A laisser en premier        */

    unsigned int shared_count;              /* Compteur de partages        */

};


/* Représentation d'un registre (classe) */
struct _GArchRegisterClass
{
    GObjectClass parent;                    /* A laisser en premier        */

    init_shared_fc init;                    /* Mise en place via interface */
    compare_shared_info_fc cmp_info;        /* Comparaison des détails     */

    reg_hash_fc hash;                       /* Production d'empreinte      */
    reg_compare_fc compare;                 /* Comparaison de registres    */
    reg_print_fc print;                     /* Impression du registre      */
    reg_is_base_pointer_fc is_bp;           /* Correspondance avec ebp     */
    reg_is_stack_pointer_fc is_sp;          /* Correspondance avec esp     */

};



/* ------------------------- REGISTRE SOUS FORME D'OPERANDE ------------------------- */


/* Définition d'un opérande visant un registre (instance) */
struct _GRegisterOperand
{
    GArchOperand parent;                    /* Instance parente            */

    GArchRegister *reg;                     /* Registre représenté         */
    bool is_written;                        /* Changement de contenu       */

};


/* Définition d'un opérande visant un registre (classe) */
struct _GRegisterOperandClass
{
    GArchOperandClass parent;               /* Classe parente              */

};



#endif  /* _ARCH_REGISTER_INT_H */
